#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>

Adafruit_MPU6050 mpu;

float alfa;
//переменные для хранения портов куда подключен драйвер
int E1 = 6;
int IN1 = 4;
int IN2 = 5;

int E2 = 10;
int IN3 = 8;
int IN4 = 9;

//если мотор вращается не в ту сторону - поставить 1
const int inv1 = 0;
const int inv2 = 0;

//коэффициенты мин и макс скорости вращения колес
int minOut = -255;
int maxOut = 255;

//коэффициенты для ПИД-регулятора
//(Пропорциональный Интегральный Дифференциальный и время)
float kp = 10;
float T = 0.2;
float dt = 0.01;
float ki = kp / T * 2 * dt;
float kd = kp * T / 8 / dt;

//функция для расчета скорости для балансировки
int PID()
{
  //расчет отклонения (ошибки) от нормы (0 градусов)
  float err = 0 - alfa;

  //расчет интеграла (суммы значений отклонений) и отклонения на предыдущем шаге
  static float integral = 0, prevErr = 0;
  integral = constrain(integral + (float)err * dt, minOut, maxOut);
  //constrain - ограничения значения между minOut и maxOut
  //D -  расчет производной (скорость изменения угла)
  float D = (err - prevErr) / dt;
  //обновляем значение отклонения от 0
  prevErr = err;
  //выдаем ответ с какой скоростью (0-255) и в какую сторону (+-) крутить моторы
  return constrain(err * kp + integral + D * kd, minOut, maxOut);
}

//функция для вращения моторов
void go(int spd)
{
  //выводим на экран текущую скорость
  Serial.println(spd);
  analogWrite(E1, map(abs(spd),0, 255, 100, 255));
  analogWrite(E2, map(abs(spd),0, 255, 100, 255));

  if (spd > 0)
  {
    digitalWrite(IN1, 1 * pow(-1, inv1));
    digitalWrite(IN2, 1 - pow(-1, inv1));

    digitalWrite(IN3, 1 * pow(-1, inv2));
    digitalWrite(IN4, 1 - pow(-1, inv2));
  }
  else
  {
    digitalWrite(IN1, 1 - pow(-1, inv1));
    digitalWrite(IN2, 1 * pow(-1, inv1));

    digitalWrite(IN3, 1 - pow(-1, inv2));
    digitalWrite(IN4, 1 * pow(-1, inv2));
  }
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");
  mpu.setAccelerometerRange(MPU6050_RANGE_2_G);
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  mpu.setFilterBandwidth(MPU6050_BAND_260_HZ);
}

void loop() {
  // put your main code here, to run repeatedly:
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);

  alfa = atan(a.acceleration.x / a.acceleration.z) * 180 / 3.14;

  Serial.println(alfa);
  go(PID);
  delay(1000*dt);
}
